/*
 Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

 Author: Michael Tross <mt@aizo.ag>

 This file is part of dS Assistant.

 dS-Assistant is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 dS-Assistant is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

 NOTE: all APIs, especially the JSON API, offered by this application
 are subject to change without further notice.
 */

#ifndef __TRAFFIC_DECODE_H__
#define __TRAFFIC_DECODE_H__

#include <digitalSTROM/ds.h>
#include <string>
#include <map>
#include <time.h>

#include "hwdesc.h"

namespace dSA
{

struct dsutil
{

    /**
     * FID (Function ID):
     * - Bits 15..12 (FID Class)
     * - Bits 11..6 (FID Subclass)
     * - Bits 5..0 (Function Module)
     */

    #define FidGetClass(fid_)           ((fid_) >> 12)
    #define FidGetSubclass(fid_)        (((fid_) >> 6) & 0x3F)
    #define FidGetFunctionModule(fid_)  ((fid_) & 0x3F)
    #define FidSetClass(class_)         ((class_) << 12)
    #define FidSetSubclass(subclass_)   ((subclass_) << 6)

    typedef struct
    {
        //various
        uint8_t ver; // version of data set
        uint8_t len; // length of data set
        uint8_t chiptemp; // chip temperature
        uint8_t pol; // polarity of downstream receiver
        uint8_t scene; // actual scene
        uint8_t lastscene; // last scene

        //error
        uint16_t OverTempCnt; // Overtemperature event counter
        uint16_t OverCurrCnt; // Overcurrent (shortcirc) event counter
        uint16_t OverLoadCnt; // Overload event counter
        uint16_t NetOverLoadCnt; // Net overload event counter

        //upstream
        uint16_t RcMsgLongCnt; // Count of transmitted long rev chan messages
        uint16_t RcMsgShortCnt; // Count of transmitted short rev chan messages

        //downstream
        uint32_t FcHalfWaves; // #Halfwaves
        uint16_t FcStartLongCnt; // #Start-words long
        uint16_t FcStartShortCnt; // #Start-words short
        uint16_t FcMsgLongCnt; // #correct long
        uint16_t FcMsgShortCnt; // #correct short
        uint16_t FcErr10Cnt; // #corrected bits/hamming codewords (corrected from 1 to 0)
        uint16_t FcErr01Cnt; // #corrected bits/hamming codewords (corrected from 0 to 1)
        uint16_t FcErr2Cnt; // #discarded hamming codewords
        uint8_t FcBitLen[6]; // average bit length from within correct messages only excl. start word (01Beg, 01End, 10Beg, 10End, 11Beg, 11End)
        uint16_t FcBitVar[6]; // variance of bit length from within correct messages only excl. start word (01Beg, 01End, 10Beg, 10End, 11Beg, 11End)
        uint16_t FcRecToBeg; // #of receiver timeouts (beginning of hw)
        uint16_t FcRecToEnd; // #of receiver timeouts (end of hw)
        uint16_t FcRecErrBeg; // #of receiver errors (beginning of hw)
        uint16_t FcRecErrEnd; // #of receiver errors (end of hw)
        uint16_t FcBitNoMsgBeg; // #of received pulses (non-zeros) not within correctly decoded telegrams (beginning of hw)
        uint16_t FcBitNoMsgEnd; // #of received pulses (non-zeros) not within correctly decoded telegrams (end of hw)
        uint16_t FcBitMsgBeg; // #of received pulses (non-zeros) from within correctly decoded telegrams (beginning of hw)
        uint16_t FcBitMsgEnd; // #of received pulses (non-zeros) from within correctly decoded telegrams (end of hw)
    }__attribute__ ((packed)) devStats;

    static std::string hkDecodeShort(uint8_t msg);
    static std::string hkDecodeLong0(uint8_t msg);
    static std::string hkDecodeLong8(uint8_t msg, uint8_t data);
    static std::string hkDecodeLong16(uint8_t msg, uint16_t data);
    static std::string rkDecodeShort(dsuid_t src_dsm_dsuid, uint16_t device_id,
            uint8_t is_resend, uint8_t is_sensor,
            uint8_t key_type, uint8_t click_type,
            uint8_t flags, uint8_t cflags);
    static std::string rkDecodeLong(dsuid_t src_dsm_dsuid, uint16_t device_id,
            uint8_t mainqualifier, uint8_t subqualifier, uint32_t data,
            uint8_t flags, uint8_t cflags);
};

}; // namespace

#endif//__TRAFFIC_DECODE_H__
